(function (Drupal, drupalSettings, klaro) {

  'use strict';

  /**
   * Initialize Klaro! with custom settings.
   */
  Drupal.behaviors.klaro = {
    config: false,
    manager: false,
    observing: false,
    attach: function (context, settings) {
      if (!settings.klaro || !klaro) {
        return;
      }

      // Bootstrap config once.
      if (!Drupal.behaviors.klaro.config) {
        settings.klaro.config.services.forEach(function (service, a_index) {
          // Set callback for each service.
          settings.klaro.config.services[a_index].callback = Drupal.behaviors.klaro.serviceCallback;
          // Create regular expressions from config.
          service.cookies.forEach(function (cookie, c_index) {
            settings.klaro.config.services[a_index]['cookies'][c_index] = [new RegExp(cookie[0], 'i'), cookie[1] || '/', cookie[2] || location.host];
          });
        });

        settings.klaro.config.autoFocus = true;
        Drupal.behaviors.klaro.config = settings.klaro.config;
      }

      // Setup klaro on each attach (support ajax insert commands).
      klaro.setup(Drupal.behaviors.klaro.config);

      // Observe klaro-element to add aria-features.
      Drupal.behaviors.klaro.klaroElementObserver();

      // Fix broken aria reference, see #3483896
      document.querySelector('#klaro-cookie-notice')?.removeAttribute('aria-labelledby');

      // Store reference to manager once.
      if (!Drupal.behaviors.klaro.manager) {
        Drupal.behaviors.klaro.manager = klaro.getManager(Drupal.behaviors.klaro.config);
      }

      if(context === document) {
        var elements = once('klaro', 'body', context);
        Array.prototype.forEach.call(elements, function () {
          // Add toggle dialog button.
          if (settings.klaro.show_toggle_button && !document.getElementById('klaro_toggle_dialog')) {
            var button_label = Drupal.t("Manage consents", {},{context: 'klaro'})
            var button_html = '<button id="klaro_toggle_dialog" aria-label="' + button_label + '" aria-haspopup="dialog" title="'+ button_label + '" type="button" class="klaro_toggle_dialog klaro_toggle_dialog_override" rel="open-consent-manager"></button>';
            document.body.insertAdjacentHTML('afterbegin', button_html);
          }
        });
      }
      else {
        for (var app in Drupal.behaviors.klaro.manager.executedOnce) {
          if (Drupal.behaviors.klaro.manager.executedOnce[app]) {
            // (Re-)apply consents on each attach (ajax add_js support)
            // only if there are elements with data-src left.
            if (document.querySelectorAll('[data-name="' + app + '"][data-src]:not([src])').length > 0) {
              Drupal.behaviors.klaro.manager.applyConsents();
            }
          }
        }

      }
    },

    /**
     * A callback for each service that will run at the beginning and on consent change.
     */
    serviceCallback: function (consent, service) {

      // If onlyOnce, abort if already executed.
      if (Drupal.behaviors.klaro.manager && service.onlyOnce && Drupal.behaviors.klaro.manager.executedOnce[service.name]) {
        return;
      }

      // Execute callbackCode from config form.
      if (service.callbackCode.length > 0) {
        Function('consent', 'service', service.callbackCode)(consent, service);
      }

      // Do the behaviors only for consent.
      if (!consent) {
        return;
      }

      let knownBehaviors = Object.keys(Drupal.behaviors);
      var elements = document.querySelectorAll('[data-name="' + service.name + '"][src]');
      if (!elements) {
        return;
      }

      // #3459261 Fix HtmlMediaElements not loading correctly.
      var source_tags = document.querySelectorAll('source[data-name="' + service.name + '"]');
      if (source_tags.length > 0) {
        var loading_state;
        for (var i = 0; i < source_tags.length; i++) {
          loading_state = source_tags[i].parentElement.readyState;
          if (loading_state == 0 || loading_state == 1) {
            source_tags[i].parentElement.load();
          }
        }
      }

      function processBehaviors() {
        for (let name in Drupal.behaviors) {
          if (knownBehaviors.indexOf(name) === -1) {
            // Execute newly added behaviors. This is somewhat guesswork, as
            // the order of execution is undetermined based on file load order.
            Drupal.behaviors[name].attach(document, drupalSettings);
            knownBehaviors.push(name);
          }
        }
      }

      Array.prototype.forEach.call(elements, function(element) {
        if (element.complete) {
          processBehaviors();
        }
        else {
          element.addEventListener('load', processBehaviors);
        }
      });
    },

    /**
     * Watch for DOMchanges on klaro-element.
     */
    klaroElementObserver: function() {
      var targetNode = document.getElementById("klaro");
      if (Drupal.behaviors.klaro.observing || !targetNode) {
        return;
      }
      Drupal.behaviors.klaro.observing = true;

      const config = { childList: true, subtree: true };
      const callback = (mutationList, observer) => {
        for (const mutation of mutationList) {
          if (mutation.type === "childList") {
            Drupal.behaviors.klaro.klaroElementMutator();
          }
        }
      };
      // Create an observer instance linked to the callback function
      const observer = new MutationObserver(callback);
      observer.observe(targetNode, config);
      Drupal.behaviors.klaro.klaroElementMutator();
    },

    /**
     * Add minimal accessibility features to the customize consent dialog.
     */
    klaroElementMutator: function() {
      var labels = document.querySelectorAll('#klaro label')
      if (labels.length < 1){
        return;
      }
      for (var i = 0; i < labels.length; i++) {
        labels[i].setAttribute('tabindex', '0');
        labels[i].setAttribute('onkeydown', 'Drupal.behaviors.klaro.KlaroToggleService(event)');
        labels[i].setAttribute('aria-role', 'checkbox');
        labels[i].setAttribute('aria-labelledby', labels[i].getAttribute('for') + '-title');
        labels[i].setAttribute('aria-describedby', labels[i].getAttribute('for') + '-description');
        if (labels[i].previousElementSibling && labels[i].previousElementSibling.tagName == 'INPUT') {
          labels[i].setAttribute('aria-checked', labels[i].previousElementSibling.checked);
        }
      }
      labels[0].focus();
    },
    KlaroToggleService: function(event) {
      if (event.key === "Enter") {
        event.target.click();
        event.target.focus();
        event.target.setAttribute('aria-checked', event.target.previousElementSibling.checked);
      }
    }
  };

  /**
   * Open the Klaro! consent notice from links & buttons.
   */
  Drupal.behaviors.klaroLink = {
    attach: function (context, settings) {
      var elements = once('klaroLink', '[rel*="open-consent-manager"]', context);
      Array.prototype.forEach.call(elements, function(element) {
        element.addEventListener('click', function (event) {
          klaro.show(Drupal.behaviors.klaro.config);
          event.preventDefault();
          return false;
        });
      });
    }
  };
})(Drupal, drupalSettings, klaro);
